"""create page table

Revision ID: 2cb79bf80b46
Revises: 1fa4469fb63b
Create Date: 2013-11-13 10:23:53.306336

"""

# revision identifiers, used by Alembic.
revision = '2cb79bf80b46'
down_revision = '1fa4469fb63b'

from alembic import op
import sqlalchemy as sa


def upgrade():
    pass


def downgrade():
    pass
